@@ -15,6 +15,8 @@
  */
 package org.gradle.internal.reflect;
 
+import org.gradle.api.reflect.ObjectInstantiationException;
+
 /**
  * An object that can create new instances of a given type, which may be decorated in some fashion.
  */
@@ -25,6 +27,6 @@ public interface Instantiator {
      *
      * @throws ObjectInstantiationException On failure to create the new instance.
      */
-    <T> T newInstance(Class<T> type, Object... parameters) throws ObjectInstantiationException;
+    <T> T newInstance(Class<? extends T> type, Object... parameters) throws ObjectInstantiationException;
 
-}
\ No newline at end of file
+}
