@@ -15,18 +15,73 @@
  */
 package org.gradle.util;
 
+import com.google.common.base.Joiner;
+import com.google.common.base.Preconditions;
+import com.google.common.collect.ImmutableList;
+import com.google.common.collect.ImmutableListMultimap;
 import org.gradle.api.Action;
 import org.gradle.api.Transformer;
-import org.gradle.api.internal.Transformers;
 import org.gradle.api.specs.Spec;
+import org.gradle.internal.Cast;
+import org.gradle.internal.Factory;
+import org.gradle.internal.Pair;
+import org.gradle.internal.Transformers;
 
+import javax.annotation.Nullable;
 import java.lang.reflect.Array;
-import java.util.*;
+import java.util.AbstractList;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collection;
+import java.util.Collections;
+import java.util.Comparator;
+import java.util.Enumeration;
+import java.util.HashMap;
+import java.util.HashSet;
+import java.util.Iterator;
+import java.util.LinkedHashMap;
+import java.util.LinkedHashSet;
+import java.util.LinkedList;
+import java.util.List;
+import java.util.Map;
+import java.util.NoSuchElementException;
+import java.util.Set;
 
-import static org.gradle.api.internal.Cast.cast;
+import static org.gradle.internal.Cast.cast;
 
 public abstract class CollectionUtils {
 
+    /**
+     * Returns null if the collection is empty otherwise expects a {@link #single(Iterable)} element to be found.
+     */
+    @Nullable
+    public static <T> T findSingle(Collection<T> source) {
+        return source.isEmpty() ? null : single(source);
+    }
+
+    /**
+     * Returns the single element in the collection or throws.
+     */
+    public static <T> T single(Iterable<? extends T> source) {
+        Iterator<? extends T> iterator = source.iterator();
+        if (!iterator.hasNext()) {
+            throw new NoSuchElementException("Expecting collection with single element, got none.");
+        }
+        T element = iterator.next();
+        if (iterator.hasNext()) {
+            throw new IllegalArgumentException("Expecting collection with single element, got multiple.");
+        }
+        return element;
+    }
+
+    public static <T> Collection<? extends T> checkedCast(Class<T> type, Collection<?> input) {
+        for (Object o : input) {
+            cast(type, o);
+        }
+        return Cast.uncheckedCast(input);
+    }
+
+    @Nullable
     public static <T> T findFirst(Iterable<? extends T> source, Spec<? super T> filter) {
         for (T item : source) {
             if (filter.isSatisfiedBy(item)) {
@@ -37,6 +92,29 @@ public abstract class CollectionUtils {
         return null;
     }
 
+    @Nullable
+    public static <T> T findFirst(T[] source, Spec<? super T> filter) {
+        for (T thing : source) {
+            if (filter.isSatisfiedBy(thing)) {
+                return thing;
+            }
+        }
+
+        return null;
+    }
+
+    public static <T> T first(Iterable<? extends T> source) {
+        return source.iterator().next();
+    }
+
+    public static <T> boolean any(Iterable<? extends T> source, Spec<? super T> filter) {
+        return findFirst(source, filter) != null;
+    }
+
+    public static <T> boolean any(T[] source, Spec<? super T> filter) {
+        return findFirst(source, filter) != null;
+    }
+
     public static <T> Set<T> filter(Set<? extends T> set, Spec<? super T> filter) {
         return filter(set, new LinkedHashSet<T>(), filter);
     }
@@ -45,10 +123,27 @@ public abstract class CollectionUtils {
         return filter(list, new LinkedList<T>(), filter);
     }
 
-    public static <T> List<T> sort(List<? extends T> list, Comparator<? super T> comparator) {
-        List<T> sortedList = new ArrayList<T>(list);
-        Collections.sort(sortedList, comparator);
-        return sortedList;
+    public static <T> List<T> filter(T[] array, Spec<? super T> filter) {
+        return filter(Arrays.asList(array), new LinkedList<T>(), filter);
+    }
+
+
+    /**
+     * Returns a sorted copy of the provided collection of things. Uses the provided comparator to sort.
+     */
+    public static <T> List<T> sort(Iterable<? extends T> things, Comparator<? super T> comparator) {
+        List<T> copy = toMutableList(things);
+        Collections.sort(copy, comparator);
+        return copy;
+    }
+
+    /**
+     * Returns a sorted copy of the provided collection of things. Uses the natural ordering of the things.
+     */
+    public static <T extends Comparable> List<T> sort(Iterable<T> things) {
+        List<T> copy = toMutableList(things);
+        Collections.sort(copy);
+        return copy;
     }
 
     public static <T, C extends Collection<T>> C filter(Iterable<? extends T> source, C destination, Spec<? super T> filter) {
@@ -75,7 +170,8 @@ public abstract class CollectionUtils {
     }
 
     public static <R, I> R[] collectArray(I[] list, Class<R> newType, Transformer<? extends R, ? super I> transformer) {
-        return collectArray(list, (R[]) Array.newInstance(newType, list.length), transformer);
+        @SuppressWarnings("unchecked") R[] destination = (R[]) Array.newInstance(newType, list.length);
+        return collectArray(list, destination, transformer);
     }
 
     public static <R, I> R[] collectArray(I[] list, R[] destination, Transformer<? extends R, ? super I> transformer) {
@@ -98,6 +194,10 @@ public abstract class CollectionUtils {
         return collect(set, new HashSet<R>(), transformer);
     }
 
+    public static <R, I> List<R> collect(Iterable<? extends I> source, Transformer<? extends R, ? super I> transformer) {
+        return collect(source, new LinkedList<R>(), transformer);
+    }
+
     public static <R, I, C extends Collection<R>> C collect(Iterable<? extends I> source, C destination, Transformer<? extends R, ? super I> transformer) {
         for (I item : source) {
             destination.add(transformer.transform(item));
@@ -117,8 +217,8 @@ public abstract class CollectionUtils {
      * @param things The things to flatten
      * @return A flattened list of the given things
      */
-    public static List<?> flattenToList(Object... things) {
-        return flattenToList(Object.class, things);
+    public static List<?> flattenCollections(Object... things) {
+        return flattenCollections(Object.class, things);
     }
 
     /**
@@ -132,7 +232,7 @@ public abstract class CollectionUtils {
      * @param <T> The target type in the flattened list
      * @return A flattened list of the given things
      */
-    public static <T> List<T> flattenToList(Class<T> type, Object... things) {
+    public static <T> List<T> flattenCollections(Class<T> type, Object... things) {
         if (things == null) {
             return Collections.singletonList(null);
         } else if (things.length == 0) {
@@ -144,20 +244,23 @@ public abstract class CollectionUtils {
                 return Collections.singletonList(null);
             }
 
+            // Casts to Class below are to workaround Eclipse compiler bug
+            // See: https://github.com/gradle/gradle/pull/200
+
             if (thing.getClass().isArray()) {
                 Object[] thingArray = (Object[]) thing;
                 List<T> list = new ArrayList<T>(thingArray.length);
                 for (Object thingThing : thingArray) {
-                    list.addAll(flattenToList(type, thingThing));
+                    list.addAll(flattenCollections(type, thingThing));
                 }
                 return list;
             }
 
-            if (thing instanceof Iterable) {
-                Iterable<?> iterableThing = (Iterable<?>) thing;
+            if (thing instanceof Collection) {
+                Collection<?> collection = (Collection<?>) thing;
                 List<T> list = new ArrayList<T>();
-                for (Object thingThing : iterableThing) {
-                    list.addAll(flattenToList(type, thingThing));
+                for (Object element : collection) {
+                    list.addAll(flattenCollections(type, element));
                 }
                 return list;
             }
@@ -166,20 +269,32 @@ public abstract class CollectionUtils {
         } else {
             List<T> list = new ArrayList<T>();
             for (Object thing : things) {
-                list.addAll(flattenToList(type, thing));
+                list.addAll(flattenCollections(type, thing));
             }
             return list;
         }
     }
 
     public static <T> List<T> toList(Iterable<? extends T> things) {
+        if (things instanceof List) {
+            @SuppressWarnings("unchecked") List<T> castThings = (List<T>) things;
+            return castThings;
+        }
+        return toMutableList(things);
+    }
+
+    public static <T> List<T> toList(Enumeration<? extends T> things) {
+        AbstractList<T> list = new ArrayList<T>();
+        while (things.hasMoreElements()) {
+            list.add(things.nextElement());
+        }
+        return list;
+    }
+
+    private static <T> List<T> toMutableList(Iterable<? extends T> things) {
         if (things == null) {
             return new ArrayList<T>(0);
         }
-        if (things instanceof List) {
-            return (List<T>) things;
-        }
-
         List<T> list = new ArrayList<T>();
         for (T thing : things) {
             list.add(thing);
@@ -187,12 +302,39 @@ public abstract class CollectionUtils {
         return list;
     }
 
+
+    public static <T> List<T> intersection(Collection<? extends Collection<T>> availableValuesByDescriptor) {
+        List<T> result = new ArrayList<T>();
+        Iterator<? extends Collection<T>> iterator = availableValuesByDescriptor.iterator();
+        if (iterator.hasNext()) {
+            Collection<T> firstSet = iterator.next();
+            result.addAll(firstSet);
+            while (iterator.hasNext()) {
+                Collection<T> next = iterator.next();
+                result.retainAll(next);
+            }
+        }
+        return result;
+
+    }
+
+    public static <T> List<T> toList(T[] things) {
+        if (things == null || things.length == 0) {
+            return new ArrayList<T>(0);
+        }
+
+        List<T> list = new ArrayList<T>(things.length);
+        Collections.addAll(list, things);
+        return list;
+    }
+
     public static <T> Set<T> toSet(Iterable<? extends T> things) {
         if (things == null) {
             return new HashSet<T>(0);
         }
         if (things instanceof Set) {
-            return (Set<T>) things;
+            @SuppressWarnings("unchecked") Set<T> castThings = (Set<T>) things;
+            return castThings;
         }
 
         Set<T> set = new LinkedHashSet<T>();
@@ -229,7 +371,7 @@ public abstract class CollectionUtils {
         return collect(source, destination, Transformers.asString());
     }
 
-    public static List<String> stringize(List<?> source) {
+    public static List<String> stringize(Collection<?> source) {
         return stringize(source, new ArrayList<String>(source.size()));
     }
 
@@ -252,12 +394,30 @@ public abstract class CollectionUtils {
         }
     }
 
+    /**
+     * Given a set of values, derive a set of keys and return a map
+     */
     public static <K, V> Map<K, V> collectMap(Iterable<? extends V> items, Transformer<? extends K, ? super V> keyGenerator) {
         Map<K, V> map = new LinkedHashMap<K, V>();
         collectMap(map, items, keyGenerator);
         return map;
     }
 
+    public static <K, V> void collectMapValues(Map<K, V> destination, Iterable<? extends K> keys, Transformer<? extends V, ? super K> keyGenerator) {
+        for (K item : keys) {
+            destination.put(item, keyGenerator.transform(item));
+        }
+    }
+
+    /**
+     * Given a set of keys, derive a set of values and return a map
+     */
+    public static <K, V> Map<K, V> collectMapValues(Iterable<? extends K> keys, Transformer<? extends V, ? super K> keyGenerator) {
+        Map<K, V> map = new LinkedHashMap<K, V>();
+        collectMapValues(map, keys, keyGenerator);
+        return map;
+    }
+
     public static <T> boolean every(Iterable<? extends T> things, Spec<? super T> predicate) {
         for (T thing : things) {
             if (!predicate.isSatisfiedBy(thing)) {
@@ -283,6 +443,19 @@ public abstract class CollectionUtils {
         return t1;
     }
 
+    /**
+     * Utility for adding an array to a collection.
+     *
+     * @param t1 The collection to add to
+     * @param t2 The iterable to add each item of to the collection
+     * @param <T> The element type of t1
+     * @return t1
+     */
+    public static <T> Collection<T> addAll(Collection<T> t1, T... t2) {
+        Collections.addAll(t1, t2);
+        return t1;
+    }
+
     /**
      * The result of diffing two sets.
      *
@@ -290,13 +463,8 @@ public abstract class CollectionUtils {
      * @see CollectionUtils#diffSetsBy(java.util.Set, java.util.Set, org.gradle.api.Transformer)
      */
     public static class SetDiff<T> {
-        public static class Pair<T> {
-            public T left;
-            public T right;
-        }
-
         public Set<T> leftOnly = new HashSet<T>();
-        public Set<Pair<T>> common = new HashSet<Pair<T>>();
+        public Set<Pair<T, T>> common = new HashSet<Pair<T, T>>();
         public Set<T> rightOnly = new HashSet<T>();
     }
 
@@ -331,9 +499,7 @@ public abstract class CollectionUtils {
             if (rightValue == null) {
                 setDiff.leftOnly.add(leftEntry.getValue());
             } else {
-                SetDiff.Pair<T> pair = new SetDiff.Pair<T>();
-                pair.left = leftEntry.getValue();
-                pair.right = rightValue;
+                Pair<T, T> pair = Pair.of(leftEntry.getValue(), rightValue);
                 setDiff.common.add(pair);
             }
         }
@@ -389,18 +555,42 @@ public abstract class CollectionUtils {
             throw new NullPointerException("The 'objects' cannot be null");
         }
 
-        boolean first = true;
         StringBuilder string = new StringBuilder();
-        for (Object object : objects) {
-            if (!first) {
+        Iterator<?> iterator = objects.iterator();
+        if (iterator.hasNext()) {
+            string.append(iterator.next().toString());
+            while (iterator.hasNext()) {
                 string.append(separator);
+                string.append(iterator.next().toString());
             }
-            string.append(object.toString());
-            first = false;
         }
         return string.toString();
     }
 
+    /**
+     * Partition given Collection into a Pair of Collections.
+     *
+     * <pre>Left</pre> Collection containing entries that satisfy the given predicate
+     * <pre>Right</pre> Collection containing entries that do NOT satisfy the given predicate
+     */
+    public static <T> Pair<Collection<T>, Collection<T>> partition(Iterable<T> items, Spec<? super T> predicate) {
+        Preconditions.checkNotNull(items, "Cannot partition null Collection");
+        Preconditions.checkNotNull(predicate, "Cannot apply null Spec when partitioning");
+
+        Collection<T> left = new LinkedList<T>();
+        Collection<T> right = new LinkedList<T>();
+
+        for (T item : items) {
+            if (predicate.isSatisfiedBy(item)) {
+                left.add(item);
+            } else {
+                right.add(item);
+            }
+        }
+
+        return Pair.of(left, right);
+    }
+
     public static class InjectionStep<T, I> {
         private final T target;
         private final I item;
@@ -436,4 +626,46 @@ public abstract class CollectionUtils {
         return target;
     }
 
-}
\ No newline at end of file
+    public static <K, V> Map<K, Collection<V>> groupBy(Iterable<? extends V> iterable, Transformer<? extends K, V> grouper) {
+        ImmutableListMultimap.Builder<K, V> builder = ImmutableListMultimap.builder();
+
+        for (V element : iterable) {
+            K key = grouper.transform(element);
+            builder.put(key, element);
+        }
+
+        return builder.build().asMap();
+    }
+
+    public static <T> Iterable<? extends T> unpack(final Iterable<? extends Factory<? extends T>> factories) {
+        return new Iterable<T>() {
+            private final Iterator<? extends Factory<? extends T>> delegate = factories.iterator();
+
+            public Iterator<T> iterator() {
+                return new Iterator<T>() {
+                    public boolean hasNext() {
+                        return delegate.hasNext();
+                    }
+
+                    public T next() {
+                        return delegate.next().create();
+                    }
+
+                    public void remove() {
+                        throw new UnsupportedOperationException();
+                    }
+                };
+            }
+        };
+    }
+
+    @Nullable
+    public static <T> List<T> nonEmptyOrNull(Iterable<T> iterable) {
+        ImmutableList<T> list = ImmutableList.copyOf(iterable);
+        return list.isEmpty() ? null : list;
+    }
+
+    public static String asCommandLine(Iterable<String> arguments) {
+        return Joiner.on(" ").join(collect(arguments, Transformers.asSafeCommandLineArgument()));
+    }
+}
