@@ -16,7 +16,9 @@
 
 package org.gradle.launcher.daemon.protocol;
 
-import java.io.Serializable;
-
-public class Finished implements Serializable {
+/**
+ * Sent from the daemon client to the daemon to indicate it has finished with the connection. This is the last
+ * message sent from the client to the daemon.
+ */
+public class Finished extends Message {
 }
