@@ -6,7 +6,7 @@ public class ${testClassName} {
 
     private final ${productionClassName} production = new ${productionClassName}("value");
 
-<% 20.times { index ->  %>
+<% (binding.hasVariable("testMethodCount") ? testMethodCount : 20).times { index ->  %>
     @org.junit.Test
     public void test${index}() {
         assertEquals(production.getProperty(), "value");
