@@ -16,12 +16,9 @@
 
 package org.gradle.tooling.internal.consumer;
 
-import org.gradle.listener.ListenerManager;
-import org.gradle.logging.ProgressLoggerFactory;
+import org.gradle.internal.event.ListenerManager;
+import org.gradle.internal.logging.progress.ProgressLoggerFactory;
 
-/**
- * by Szczepan Faber, created at: 12/14/11
- */
 public interface LoggingProvider {
     ListenerManager getListenerManager();
     ProgressLoggerFactory getProgressLoggerFactory();
